@@ -29,6 +29,7 @@ fn main() {
     opts.optmulti("", "ignore", "Exclude a device name or path", "NAME");
     opts.optflag("", "watch", "Add new devices automatically");
     opts.optflag("h", "help", "print this help menu");
+    opts.optflag("", "version", "show version");
 
     let args = match opts.parse(&argv[1..]) {
         Ok(args) => args,
@@ -38,6 +39,10 @@ fn main() {
         println!("{}", &usage(&program, opts));
         return;
     }
+    if args.opt_present("version") {
+        println!("xremap version {}", env!("CARGO_PKG_VERSION"));
+        return;
+    }
 
     let filename = match &args.free.iter().map(String::as_str).collect::<Vec<&str>>()[..] {
         &[filename] => filename,
